@@ -5,14 +5,12 @@
 #include <stdio.h>
 #include <errno.h>
 #include <string.h>
-#include <asm/page.h>
 #include "selinux_internal.h"
 #include "policy.h"
 #include <limits.h>
 
 int security_compute_user_raw(security_context_t scon,
-                              const char *user,
-                              security_context_t **con)
+			      const char *user, security_context_t ** con)
 {
 	char path[PATH_MAX];
 	char **ary;
@@ -21,12 +19,17 @@ int security_compute_user_raw(security_context_t scon,
 	int fd, ret;
 	unsigned int i, nel;
 
+	if (!selinux_mnt) {
+		errno = ENOENT;
+		return -1;
+	}
+
 	snprintf(path, sizeof path, "%s/user", selinux_mnt);
 	fd = open(path, O_RDWR);
 	if (fd < 0)
 		return -1;
 
-	size = PAGE_SIZE;
+	size = selinux_page_size;
 	buf = malloc(size);
 	if (!buf) {
 		ret = -1;
@@ -35,11 +38,11 @@ int security_compute_user_raw(security_context_t scon,
 	snprintf(buf, size, "%s %s", scon, user);
 
 	ret = write(fd, buf, strlen(buf));
-	if (ret < 0) 
+	if (ret < 0)
 		goto out2;
 
 	memset(buf, 0, size);
-	ret = read(fd, buf, size-1);
+	ret = read(fd, buf, size - 1);
 	if (ret < 0)
 		goto out2;
 
@@ -48,7 +51,7 @@ int security_compute_user_raw(security_context_t scon,
 		goto out2;
 	}
 
-	ary = malloc((nel+1)*sizeof(char*));
+	ary = malloc((nel + 1) * sizeof(char *));
 	if (!ary) {
 		ret = -1;
 		goto out2;
@@ -67,42 +70,41 @@ int security_compute_user_raw(security_context_t scon,
 	ary[nel] = NULL;
 	*con = ary;
 	ret = 0;
-out2:
+      out2:
 	free(buf);
-out:
+      out:
 	close(fd);
 	return ret;
 }
+
 hidden_def(security_compute_user_raw)
 
 int security_compute_user(security_context_t scon,
-                          const char *user,
-                          security_context_t **con)
+			  const char *user, security_context_t ** con)
 {
 	int ret;
 	security_context_t rscon = scon;
 
-	if (context_translations && trans_to_raw_context(scon, &rscon))
+	if (selinux_trans_to_raw_context(scon, &rscon))
 		return -1;
 
- 	ret = security_compute_user_raw(rscon, user, con);
+	ret = security_compute_user_raw(rscon, user, con);
 
-	if (context_translations) {
-		freecon(rscon);
-		if (!ret) {
-			security_context_t *ptr, tmpcon;
-			for (ptr = *con; *ptr; ptr++) {
-				if (raw_to_trans_context(*ptr, &tmpcon)) {
-					freeconary(*con);
-					*con = NULL;
-					return -1;
-				}
-				freecon(*ptr);
-				*ptr = tmpcon;
+	freecon(rscon);
+	if (!ret) {
+		security_context_t *ptr, tmpcon;
+		for (ptr = *con; *ptr; ptr++) {
+			if (selinux_raw_to_trans_context(*ptr, &tmpcon)) {
+				freeconary(*con);
+				*con = NULL;
+				return -1;
 			}
+			freecon(*ptr);
+			*ptr = tmpcon;
 		}
 	}
 
 	return ret;
 }
+
 hidden_def(security_compute_user)
