@@ -9,6 +9,7 @@
 #include <dlfcn.h>
 #include <sys/vfs.h>
 #include <stdint.h>
+#include <limits.h>
 
 #include "dso.h"
 #include "policy.h"
@@ -17,6 +18,7 @@
 
 char *selinux_mnt = NULL;
 int selinux_page_size = 0;
+int obj_class_compat = 1;
 
 static void init_selinuxmnt(void)
 {
@@ -26,6 +28,7 @@ static void init_selinuxmnt(void)
 	int rc;
 	size_t len;
 	ssize_t num;
+	int exists = 0;
 
 	if (selinux_mnt)
 		return;
@@ -42,6 +45,23 @@ static void init_selinuxmnt(void)
 		}
 	} 
 
+	/* Drop back to detecting it the long way. */
+	fp = fopen("/proc/filesystems", "r");
+	if (!fp)
+		return;
+
+	__fsetlocking(fp, FSETLOCKING_BYCALLER);
+	while ((num = getline(&buf, &len, fp)) != -1) {
+		if (strstr(buf, "selinuxfs")) {
+			exists = 1;
+			break;
+		}
+	}
+	fclose(fp);
+
+	if (!exists)
+		return;
+
 	/* At this point, the usual spot doesn't have an selinuxfs so
 	 * we look around for it */
 	fp = fopen("/proc/mounts", "r");
@@ -92,12 +112,10 @@ static void init_lib(void)
 {
 	selinux_page_size = sysconf(_SC_PAGE_SIZE);
 	init_selinuxmnt();
-	init_context_translations();
 }
 
 static void fini_lib(void) __attribute__ ((destructor));
 static void fini_lib(void)
 {
 	fini_selinuxmnt();
-	fini_context_translations();
 }
